@@ -123,6 +123,15 @@ enum ast_frame_type {
 };
 #define AST_FRAME_DTMF AST_FRAME_DTMF_END
 
+enum {
+	/*! This frame contains valid timing information */
+	AST_FRFLAG_HAS_TIMING_INFO = (1 << 0),
+	/*! This frame came from a translator and is still the original frame.
+	 *  The translator can not be free'd if the frame inside of it still has
+	 *  this flag set. */
+	AST_FRFLAG_FROM_TRANSLATOR = (1 << 1),
+};
+
 /*! \brief Data structure associated with a single frame of data
  */
 struct ast_frame {
@@ -148,8 +157,8 @@ struct ast_frame {
 	struct timeval delivery;
 	/*! For placing in a linked list */
 	AST_LIST_ENTRY(ast_frame) frame_list;
-	/*! Timing data flag */
-	int has_timing_info;
+	/*! Misc. frame flags */
+	unsigned int flags;
 	/*! Timestamp in milliseconds */
 	long ts;
 	/*! Length in milliseconds */
@@ -419,7 +428,7 @@ void ast_swapcopy_samples(void *dst, const void *src, int samples);
 
 /*! \brief Get the name of a format
  * \param format id of format
- * \return A static string containing the name of the format or "UNKN" if unknown.
+ * \return A static string containing the name of the format or "unknown" if unknown.
  */
 char* ast_getformatname(int format);
 
@@ -496,6 +505,10 @@ void ast_codec_pref_remove(struct ast_codec_pref *pref, int format);
 */
 int ast_codec_pref_append(struct ast_codec_pref *pref, int format);
 
+/*! \brief Prepend an audio codec to a preference list, removing it first if it was already there 
+*/
+void ast_codec_pref_prepend(struct ast_codec_pref *pref, int format, int only_if_existing);
+
 /*! \brief Select the best audio format according to preference list from supplied options. 
    If "find_best" is non-zero then if nothing is found, the "Best" format of 
    the format list is selected, otherwise 0 is returned. */
