@@ -25,7 +25,7 @@
 
 #include "asterisk.h"
 
-ASTERISK_FILE_VERSION(__FILE__, "$Revision: 85315 $")
+ASTERISK_FILE_VERSION(__FILE__, "$Revision: 97194 $")
 
 #include <ctype.h>
 #include <string.h>
@@ -511,7 +511,7 @@ const char *ast_inet_ntoa(struct in_addr ia)
 #ifdef DEBUG_THREADS
 
 /*! \brief A reasonable maximum number of locks a thread would be holding ... */
-#define AST_MAX_LOCKS 16
+#define AST_MAX_LOCKS 64
 
 /* Allow direct use of pthread_mutex_t and friends */
 #undef pthread_mutex_t
@@ -704,8 +704,12 @@ static const char *locktype2str(enum ast_lock_type type)
 static int handle_show_locks(int fd, int argc, char *argv[])
 {
 	struct thr_lock_info *lock_info;
+	struct ast_dynamic_str *str;
 
-	ast_cli(fd, "\n" 
+	if (!(str = ast_dynamic_str_create(4096)))
+		return RESULT_FAILURE;
+
+	ast_dynamic_str_append(&str, 0, "\n" 
 	            "=======================================================================\n"
 	            "=== Currently Held Locks ==============================================\n"
 	            "=======================================================================\n"
@@ -713,32 +717,69 @@ static int handle_show_locks(int fd, int argc, char *argv[])
 				"=== <file> <line num> <function> <lock name> <lock addr> (times locked)\n"
 				"===\n");
 
+	if (!str)
+		return RESULT_FAILURE;
+
 	pthread_mutex_lock(&lock_infos_lock.mutex);
 	AST_LIST_TRAVERSE(&lock_infos, lock_info, entry) {
 		int i;
-		ast_cli(fd, "=== Thread ID: %u (%s)\n", (int) lock_info->thread_id,
+		ast_dynamic_str_append(&str, 0, "=== Thread ID: %u (%s)\n", (int) lock_info->thread_id,
 			lock_info->thread_name);
 		pthread_mutex_lock(&lock_info->lock);
-		for (i = 0; i < lock_info->num_locks; i++) {
-			ast_cli(fd, "=== ---> %sLock #%d (%s): %s %d %s %s %p (%d)\n", 
-				lock_info->locks[i].pending > 0 ? "Waiting for " : lock_info->locks[i].pending < 0 ? "Tried and failed to get " : "", i,
+		for (i = 0; str && i < lock_info->num_locks; i++) {
+			int j;
+			ast_mutex_t *lock;
+
+			ast_dynamic_str_append(&str, 0, "=== ---> %sLock #%d (%s): %s %d %s %s %p (%d)\n", 
+				lock_info->locks[i].pending > 0 ? "Waiting for " : 
+					lock_info->locks[i].pending < 0 ? "Tried and failed to get " : "", i,
 				lock_info->locks[i].file, 
 				locktype2str(lock_info->locks[i].type),
 				lock_info->locks[i].line_num,
 				lock_info->locks[i].func, lock_info->locks[i].lock_name,
 				lock_info->locks[i].lock_addr, 
 				lock_info->locks[i].times_locked);
+
+			if (!lock_info->locks[i].pending || lock_info->locks[i].pending == -1)
+				continue;
+
+			/* We only have further details for mutexes right now */
+			if (lock_info->locks[i].type != AST_MUTEX)
+				continue;
+
+			lock = lock_info->locks[i].lock_addr;
+
+			ast_reentrancy_lock(lock);
+			for (j = 0; str && j < lock->reentrancy; j++) {
+				ast_dynamic_str_append(&str, 0, "=== --- ---> Locked Here: %s line %d (%s)\n",
+					lock->file[j], lock->lineno[j], lock->func[j]);
+			}
+			ast_reentrancy_unlock(lock);	
 		}
 		pthread_mutex_unlock(&lock_info->lock);
-		ast_cli(fd, "=== -------------------------------------------------------------------\n"
+		if (!str)
+			break;
+		ast_dynamic_str_append(&str, 0, "=== -------------------------------------------------------------------\n"
 		            "===\n");
+		if (!str)
+			break;
 	}
 	pthread_mutex_unlock(&lock_infos_lock.mutex);
 
-	ast_cli(fd, "=======================================================================\n"
+	if (!str)
+		return RESULT_FAILURE;
+
+	ast_dynamic_str_append(&str, 0, "=======================================================================\n"
 	            "\n");
 
-	return 0;
+	if (!str)
+		return RESULT_FAILURE;
+
+	ast_cli(fd, "%s", str->str);
+
+	free(str);
+
+	return RESULT_SUCCESS;
 }
 
 static char show_locks_help[] =
@@ -779,6 +820,7 @@ static void *dummy_start(void *data)
 	struct thr_arg a = *((struct thr_arg *) data);	/* make a local copy */
 #ifdef DEBUG_THREADS
 	struct thr_lock_info *lock_info;
+	pthread_mutexattr_t mutex_attr;
 #endif
 
 	/* note that even though data->name is a pointer to allocated memory,
@@ -796,7 +838,11 @@ static void *dummy_start(void *data)
 
 	lock_info->thread_id = pthread_self();
 	lock_info->thread_name = strdup(a.name);
-	pthread_mutex_init(&lock_info->lock, NULL);
+
+	pthread_mutexattr_init(&mutex_attr);
+	pthread_mutexattr_settype(&mutex_attr, AST_MUTEX_KIND);
+	pthread_mutex_init(&lock_info->lock, &mutex_attr);
+	pthread_mutexattr_destroy(&mutex_attr);
 
 	pthread_mutex_lock(&lock_infos_lock.mutex); /* Intentionally not the wrapper */
 	AST_LIST_INSERT_TAIL(&lock_infos, lock_info, entry);
@@ -920,6 +966,8 @@ char *ast_unescape_semicolon(char *s)
 		if ((e > work) && (*(e-1) == '\\')) {
 			memmove(e - 1, e, strlen(e) + 1);
 			work = e;
+		} else {
+			work = e + 1;
 		}
 	}
 
